/*!
 * Ghostery for Firefox
 * http://www.ghostery.com/
 *
 * Copyright 2013 EVIDON, Inc. All rights reserved.
 * See ghostery_eula.txt for license.
 */

var _ = require('vendor/underscore-1.4.3'),
	conf = require('conf').load,
	utils = require('utils'),
	UpdatableMixin = require('updatable').load;

var log = utils.log;

function BugDb() {
	this.type = 'bugs';

	function buildDb(bugs, version) {
		var i, num_bugs,
			bug,
			apps = {},
			cats = {},
			bugs_map = {},
			first_party_map = {},
			num_selected_patterns = 0,
			patterns_arr = [],
			priority,
			regexes = {
				high: {},
				regular: {},
				low: {}
			};

		// TODO review this code
		// Remove any existing user-created bugs
		for (i = bugs.length - 1; i > -1; i--) {
			if (bugs[i].type == 'user_created') {
				bugs.splice(i, 1);
			} else {
				break;
			}
		}

		var userBugs = processUserCreatedBugs();
		for (i = 0; i < userBugs.length; i++) {
			bugs.push(userBugs[i]);
		}

		for (i = 0, num_bugs = bugs.length; i < num_bugs; i++) {
			bug = bugs[i];

			bugs_map[bug.id] = {
				aid: bug.aid,
				name: bug.name
			};

			apps[bug.aid] = {
				name: bug.name,
				cat: bug.type
			};

			patterns_arr.push(bug.pattern);

			if (bug.hasOwnProperty('firstPartyUrls')) {
				first_party_map[bug.id] = bug.firstPartyUrls;
			}

			if (!cats.hasOwnProperty(bug.type)) {
				cats[bug.type] = {};
			}
			cats[bug.type][bug.aid] = true;

			priority = bug.priority;
			if (priority != 'high' && priority != 'low') {
				priority = 'regular';
			}
			regexes[priority][bug.id] = new RegExp(bug.pattern, 'i');

			if (conf.selected_app_ids.hasOwnProperty(bug.aid)) {
				num_selected_patterns++;
			}
		}

		return {
			JUST_UPDATED_WITH_NEW_TRACKERS: false,

			allSelected: (num_selected_patterns == patterns_arr.length),
			noneSelected: num_selected_patterns === 0,

			apps: apps,
			bugs: bugs_map,
			cats: cats,
			firstPartyExceptions: first_party_map,
			fullRegex: new RegExp(patterns_arr.join('|'), 'i'),
			regexes: regexes,
			version: version
		};
	}

	function updateNewAppIds(new_bugs, old_bugs) {
		log('updating newAppIds ...');

		var new_app_ids = _.difference(
			_.pluck(new_bugs, 'aid'),
			_.pluck(old_bugs, 'aid')
		);

		utils.prefs('newAppIds', new_app_ids);

		return new_app_ids;
	}

	function processUserCreatedBugs() {
		var inputtedBugs = [],
			userBugs = [];

		try {
			inputtedBugs = utils.prefs('user_bugs');
		} catch (e) {}

		for (var i = 0; i < inputtedBugs.length; i++) {
			var bug = inputtedBugs[i];
			if (!bug.pattern) {
				continue;
			}

			bug.userCreated = true;
			bug.type = 'user_created';

			if (!bug.hasOwnProperty('name')) {
				bug.name = 'User Created Bug ' + (i + 1);
			} else {
				bug.name = bug.name + ' (U)';
			}

			if (!bug.hasOwnProperty('aid')) {
				bug.aid = 10000 + i;
			}

			if (!bug.hasOwnProperty('cid')) {
				bug.cid = 10000 + i;
			}

			if (!bug.hasOwnProperty('id')) {
				bug.id = 10000 + i;
			}

			userBugs.push(bug);
		}

		return userBugs;
	}

	this.processList = function (bugs) {
		var db;

		log('processing bugs ...');

		try {
			db = buildDb(bugs.bugs, bugs.bugsVersion);
		} catch (e) {}

		if (!db) {
			return false;
		}

		log('processed');

		var old_bugs = utils.prefs('bugs');

		// there is an older bugs object and the versions are different
		// TODO should be doing > instead of != once we convert all versions to timestamps
		if (old_bugs && bugs.bugsVersion != old_bugs.bugsVersion) {
			var new_app_ids = updateNewAppIds(bugs.bugs, old_bugs.bugs);

			if (conf.block_by_default) {
				log('applying block-by-default ...');

				_.each(new_app_ids, function (app_id) {
					conf.selected_app_ids[app_id] = 1;
				});
			}

			if (new_app_ids.length) {
				db.JUST_UPDATED_WITH_NEW_TRACKERS = true;
			}
		}

		this.db = db;
		utils.prefs('bugs', bugs);

		return true;
	};

	_.extend(this, UpdatableMixin);
}

exports.load = new BugDb();
